@@ -1,7 +1,7 @@
 package Web::Query;
 our $AUTHORITY = 'cpan:TOKUHIROM';
 # ABSTRACT: Yet another scraping library like jQuery
-$Web::Query::VERSION = '0.31';
+$Web::Query::VERSION = '0.32';
 use strict;
 use warnings;
 use 5.008001;
@@ -12,6 +12,7 @@ use HTML::Selector::XPath 0.06 qw/selector_to_xpath/;
 use Scalar::Util qw/blessed refaddr/;
 use HTML::Entities qw/encode_entities/;
 
+use List::MoreUtils qw/ uniq /;
 use Scalar::Util qw/ refaddr /;
 our @EXPORT = qw/wq/;
 
@@ -225,8 +226,57 @@ sub text {
 
 sub attr {
     my $self = shift;
-    my @retval = map { $_->attr(@_) } @{$self->{trees}};
-    return wantarray ? @retval : $retval[0];
+
+    if ( @_ == 1 ) { # getter 
+        return wantarray 
+            ? map { $_->attr(@_) } @{$self->{trees}}
+            : eval { $self->{trees}[0]->attr(@_) }
+            ;
+    }
+
+    $_->attr(@_) for @{$self->{trees}};
+
+    return $self;
+}
+
+sub id {
+    my $self = shift;
+
+    if ( @_ ) {  # setter
+        my $new_id = shift;
+
+        return $self if $self->size == 0;
+
+        return $self->each(sub{
+            $_->attr( id => $new_id->(@_) )
+        }) if ref $new_id eq 'CODE';
+
+        if ( $self->size == 1 ) {
+            $self->attr( id => $new_id );
+        }
+        else {
+            return $self->first->attr( id => $new_id );
+        }
+    }
+    else { # getter
+
+        # the eval is there in case there is no tree
+        return wantarray 
+            ? map { $_->attr('id') } @{$self->{trees}}
+            : eval { $self->{trees}[0]->attr('id') }
+            ;
+    }
+}
+
+sub name {
+    my $self = shift;
+    $self->attr( 'name', @_ );
+}
+
+sub data {
+    my $self = shift;
+    my $name = shift;
+    $self->attr( join( '-', 'data', $name ), @_ );
 }
 
 sub tagname {
@@ -456,15 +506,28 @@ sub remove_class {
     
 }
 
+sub toggle_class {
+    my $self = shift;
+    
+    my @classes = uniq @_;
+
+    $self->each(sub{
+        for my $class ( @classes ) {
+            my $method = $_->has_class($class) ? 'remove_class' : 'add_class';
+            $_->$method($class);
+        }
+    });
+}
 
 sub has_class {
     my ($self, $class) = @_;
     
     foreach my $t (@{$self->{trees}}) {
+        no warnings 'uninitialized';
         return 1 if $t->attr('class') =~ /(?:^|\s)$class(?:\s|$)/;
     }
     
-    return;   
+    return undef;   
 }
 
 sub clone {
@@ -578,7 +641,7 @@ Web::Query - Yet another scraping library like jQuery
 
 =head1 VERSION
 
-version 0.31
+version 0.32
 
 =head1 SYNOPSIS
 
@@ -797,6 +860,23 @@ Adds the specified class(es) to each of the set of matched elements.
     # add class 'foo' to <p> elements
     wq('<div><p>foo</p><p>bar</p></div>')->find('p')->add_class('foo'); 
 
+=head3 toggle_class( @classes )
+
+Toggles the given class or classes on each of the element. I.e., if the element had the class, it'll be removed,
+and if it hadn't, it'll be added.
+
+Classes are toggled once, no matter how many times they appear in the argument list.
+
+    $q->toggle_class( 'foo', 'foo', 'bar' );
+
+    # equivalent to
+    
+    $q->toggle_class('foo')->toggle_class('bar');
+
+    # and not
+
+    $q->toggle_class('foo')->toggle_class('foo')->toggle_class('bar');
+
 =head3 after
 
 Insert content, specified by the parameter, after each element in the set of matched elements.
@@ -824,11 +904,53 @@ of the first element.
 
 =head3 C< attr >
 
-Get/Set the attribute value in element.
+Get/set the attribute value in element.
 
-    my $attr = $q->attr($name);
+In getter mode, it'll return either the values of the attribute
+for all elements of the set, or only the first one depending of the calling context.
 
-    $q->attr($name, $val);
+    my @values = $q->attr('style');      # style of all elements
+    my $first_value = $q->attr('style'); # style of first element
+
+In setter mode, it'll set the attribute value for all elements, and return back
+the original object for easy chaining.
+
+    $q->attr( 'alt' => 'a picture' )->find( ... );
+
+=head3 C< id >
+
+Get/set the elements's id attribute.
+
+In getter mode, it behaves just like C<attr()>.
+
+In setter mode, it behaves like C<attr()>, but with the following exceptions.
+
+If the attribute value is a scalar, it'll be only assigned to
+the first element of the set (as ids are supposed to be unique), and the returned object will only contain
+that first element.
+
+    my $first_element = $q->id('the_one');
+
+It's possible to set the ids of all the elements by passing a sub to C<id()>. The sub is given the same arguments as for
+C<each()>, and its return value is taken to be the new id of the elements.
+
+    $q->id( sub { my $i = shift;  'foo_' . $i } );
+
+=head3 C< name >
+
+Get/set the elements's 'name' attribute.
+
+    my $name = $q->name;  # equivalent to $q->attr( 'name' );
+
+    $q->name( 'foo' );    # equivalent to $q->attr( name => 'foo' );
+
+=head3 C< data >
+
+Get/set the elements's 'data-*name*' attributes.
+
+    my $data = $q->data('foo');  # equivalent to $q->attr( 'data-foo' );
+
+    $q->data( 'foo' => 'bar' );  # equivalent to $q->attr( 'data-foo' => 'bar' );
 
 =head3 tagname
 
